/* this is session.c file.
 * this is part of the libspopc library sources
 * copyright  2002 Benoit Rouits <brouits@free.fr>
 * released under the terms of GNU LGPL
 * (GNU Lesser General Public Licence).
 * libspopc offers simple API for a pop3 client (MTA).
 * See RFC 1725 for pop3 specifications.
 * more information on http://brouits.free.fr/libspopc/
 */

//I casted 'port' to true unsigned, now, tell me if warning continues ; #pragma warning(disable: 4761)  /* disable "integral size mismatch in argument" -warning */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <winsock.h>
#include <io.h>
#else
#include <netdb.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#endif
#include <sys/types.h>
#define socklen_t int /* actually true on most systems */

int pop3_prepare(const char* servername, const int port, struct sockaddr_in* connection, struct hostent* server){
/* prepares the pop session and returns a socket descriptor */
int sock;
#ifdef _REENTRANT
struct hostent result_buffer;
char tmp[512];
int my_error;
#endif
#ifdef WIN32

	WSADATA wsaData;

	if (WSAStartup(MAKEWORD(2, 0), &wsaData) != 0)
	{
	fprintf(stderr,"WSAStartup() failed");
	exit(1);
	}

#endif
	memset((char*)connection,0,sizeof(struct sockaddr_in));
	
#ifdef _REENTRANT
 #if defined(__sun__)||defined(HAVE_FUNC_GETHOSTBYNAME_R_5)
	server=gethostbyname_r (servername,&result_buffertmp, 512 - 1, &my_error);
 #else  /* linux */
	gethostbyname_r (servername,&result_buffer,tmp, 512 - 1, &server, &my_error);
 #endif
 #ifdef WIN32
	gethostbyname_r (servername, &result_buffer, &server);
 #endif
#else /* no _reentrant */
	server=gethostbyname(servername);
#endif
	if(!server){
		perror("pop3_prepare.gethostbyname");
		return(-1);
	}
	memmove((char*)&(connection->sin_addr.s_addr),server->h_addr,server->h_length);
	connection->sin_family=AF_INET;
	connection->sin_port=htons(port?(unsigned short int)port:(unsigned short int)110);	/* integral size mismatch in argument - htons(port)*/
	sock=socket(AF_INET,SOCK_STREAM,0);
	if(-1==sock){
		perror("pop3_prepare.socket");
		free(server);
	}
	return(sock);
}

char* pop3_connect(int sock, struct sockaddr_in* connection){
/* connects to the server through the sock and returns server's welcome */
int r;
char* buf;

#ifdef EBUG
	fprintf(stderr,"<pop3_connect>\n");
#endif
	r=connect(sock,(struct sockaddr*)connection,(socklen_t)sizeof(*connection));
	if(r==-1){
		perror("pop3_connect.connect");
		return(NULL);
	}
	buf=(char*)malloc(512);
	if(!buf){
		perror("pop3_connect.malloc");
		return(NULL);
	}
	r=recv(sock,buf,512,0); /* 512 is theorically enough, but FIXME */
	buf[r]='\0';
#ifdef EBUG
	fprintf(stderr,"recv %s",buf);
	fprintf(stderr,"</pop3_connect>\n");
#endif
	return(buf);
}

void pop3_disconnect(int sock){
/* close socket  */
	if(sock>0){
		close(sock);
	}
}
